
 { to calculate right hand side due to geopotential
   current term
   for  2 <= n <= nmax , 0 <= m <= n
    or  0 <= m <= nmax , m <= n <= nmax but n >=2
   (r_0/r)^n*Pnm(z/r)*(Cnm*cos(m*lambda)+Snm*sin(m*lambda)
   where
   Pnm(z/r)=(1-z^2/r^2)^(m/2)*(d^m(P_n(z/r))/d(z/r)^m)
   but it contains three simple parts
   the first
       (r_0/r)^n   -> only 1/r as variable but x,y,z are in r
   the second
       d^m(P_n(z/r))/d(z/r)^m  -> only z/r as variable but x,y are in r
   the third
       (1-z^2/r^2)^(m/2)*cos(m*lambda)   with Cnm coefficient
       (1-z^2/r^2)^(m/2)*sin(m*lambda)   with Snm coefficient
   we have
   (1-z^2/r^2)^(m/2) = (cos^2(phi))^(m/2) = cos^m(phi)
   where
          phi and lambda
          are spherical latitude and longitude
          in terrestrial reference system
   so
       x/r=cos(phi)*cos(lambda)
       y/r=cos(phi)*sin(lambda)
   thus we have two simple recurrent formulae
   cos^m(phi)*cos(m*lambda) =
       = cos^(m-1)(phi)*cos((m-1)*lambda)*(x/r) -
       - cos^(m-1)(phi)*sin((m-1)*lambda)*(y/r) ;
   cos^m(phi)*sin(m*lambda) =
       = cos^(m-1)(phi)*sin((m-1)*lambda)*(x/r) +
       + cos^(m-1)(phi)*cos((m-1)*lambda)*(y/r) ;
   therefore a third part is function (x/r) and (y/r) but z is in r

   corrections to geopotential coefficients
   due to solid body tides are done in procedure ToAddForceM
   by calling CoefWithTide procedure from unit UnForceT }

 UNIT UnForceH ;

 INTERFACE

 Uses
   UnConTyp ; { for TVect3 = Array[1..3] of Extended }

 Procedure ForceDueToGeoHarm ( VectXCur           : TVect3 ;
                               Var
                                 ForceCur         : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnForTyp , { global matrix SelTerM[1..3,1..3] }
   UnHarMod , { for the number of harmonics MaxIntHarm procedure ToGetC }
   UnForceT ; { for variations in geopotential coefficients due to tides }

 Type
   TMaxPow       = Array[1..MaxPowHarm+1] of Extended ;
   TMaxPowPtr    = ^TMaxPow ;

 Var
   nhcs         : Word ; { a count in records AllHarmCoef^[.] UnHarMod }
   rs,xs,ys,zs  : Extended ;
   rr,xr,yr,zr  : Extended ;
   dr,dx,dy,dz  : TVect3 ; { derivatives for 1/r x/r y/r z/r on x y z }
   xam,xax,xay  : Extended ; { for cos^m(phi)*(cos or sin)(m*lambda) }
   yam,yax,yay  : Extended ; { with derivative with respect to x/r , y/r }
   pl,ul,rl     : TMaxPowPtr ; { for Legandre polinoms and r0/r pointer }

 { position of the satellite
   from celestial to terrestrial
   by using transformation matrix  SelTerM(3,3) }

 Procedure ToTransToTerra ( VectXCur  : TVect3 ) ;
 Begin
   xs:=+SelTerM[1,1]*VectXCur[1] { to rotate from CRS to TRS }
       +SelTerM[1,2]*VectXCur[2]+SelTerM[1,3]*VectXCur[3];
   ys:=+SelTerM[2,1]*VectXCur[1] { VectXCur is vector in CRS }
       +SelTerM[2,2]*VectXCur[2]+SelTerM[2,3]*VectXCur[3];
   zs:=+SelTerM[3,1]*VectXCur[1] { xs ys zs is vector in TRS }
       +SelTerM[3,2]*VectXCur[2]+SelTerM[3,3]*VectXCur[3];
   rs:=Sqrt(1/(Sqr(xs)+Sqr(ys)+Sqr(zs))); { 1/r }
 End;

 { to calculate partial derivatives from
   1/r x/r y/r z/r  with respect to x y z }

 Procedure ToClcSimpleDer ;
 Begin
   xr:=xs*rs; { x/r , xs rs are known }
   yr:=ys*rs; { y/r , ys rs are known }
   zr:=zs*rs; { z/r , zs rs are known }
   rr:=rs*rs; { 1/r^2 , rs is known }
   dr[1]:=-xr*rr; { -x/r^3 }
   dr[2]:=-yr*rr; { -y/r^3 }
   dr[3]:=-zr*rr; { -z/r^3 }
   dx[1]:=rs+xs*dr[1]; { 1/r-x^2/r^3 }
   dx[2]:=xs*dr[2]; { -xy/r^3 }
   dx[3]:=xs*dr[3]; { -xz/r^3 }
   dy[1]:=ys*dr[1]; { -xy/r^3 }
   dy[2]:=rs+ys*dr[2]; { 1/r-y^2/r^3 }
   dy[3]:=ys*dr[3]; { -yz/r^3 }
   dz[1]:=zs*dr[1]; { -xz/r^3 }
   dz[2]:=zs*dr[2]; { -yz/r^3 }
   dz[3]:=rs+zs*dr[3]; { 1/r-z^2/r^3 }
 End;

 { for m = 0
   initial values for recurrence process
   P_n(x)=((2n-1)/n)*x*P_(n-1)(x)-(n-1)*P_(n-2)(x)
   P_n'(x)=n*P_(n-1)(x)+x*P_(n-1)'(x)
   P_n'(x)=(2n-1)*P_(n-1)(x)+P_(n-2)'(x) }

 Procedure ToClcInitRecur ;
 Var
   n  : Byte ;
   p  : Extended ;
   q  : Extended ;
   r  : Extended ;
 Begin  { zr = z/r is known }
   p:=GeoFM*rs; { fm/r  GeoFm form UnConTyp }
   r:=GeoR0*rs; { r_0/r GeoR0 from UnConTyp }
   rl^[1]:=p*r;
   rl^[2]:=r*rl^[1];
   pl^[1]:=zr;  { P_1(x)=1 Legandre polinom m=0 }
   pl^[2]:=1.5*Sqr(zr)-0.5; { P_2(x)=3/2*x*x-1/2 Legandre polinom }
   ul^[1]:=1.0;  { derivative P_1'(x)=1 m=0 }
   ul^[2]:=3.0*zr;  { derivative P_2'(x)=3*x }
   For n:=3 To MaxIntHarm+1 Do
     Begin
       rl^[n]:=r*rl^[n-1];
       q:=n; { Legandre polinom and derivative for m=0 }
       pl^[n]:=((2*q-1)/q)*zr*pl^[n-1]-((q-1)/q)*pl^[n-2]; { P_n(x) }
       ul^[n]:=q*pl^[n-1]+zr*ul^[n-1]; { P_n'(x) the first derivative }
     End; { by using  P_n'(x)=n*P_(n-1)(x)+x*P_(n-1)'(x) }
   xam:=1.0; { initial  cos^m(phi)*cos(m*lambda) but m=0 }
   yam:=0.0; { initial  cos^m(phi)*sin(m*lambda) but m=0 }
   xax:=0.0; { d(cos^m(phi)*cos(m*lambda))/d(x/r) but m=0 }
   xay:=0.0; { d(cos^m(phi)*cos(m*lambda))/d(y/r) but m=0 }
   yax:=0.0; { d(cos^m(phi)*sin(m*lambda))/d(x/r) but m=0 }
   yay:=0.0; { d(cos^m(phi)*sin(m*lambda))/d(y/r) but m=0 }
 End;

 { to calculate 
   recurrent value of the functions for the next value m
   to obtain forces we need the value
   p_(nm)(x)=d^m(P_n(x))/dx^m and p_(nm)'(x)=d^(m+1)(P_n(x))/dx^(m+1)
   for this calculations it is convenient recurrent formula
       P_n'(x)=(2n-1)*P_(n-1)(x)+P_(n-2)'(x)   the first derivative
       P_n''(x)=(2n-1)*P_(n-1)'(x)+P_(n-2)''(x)  the second derivative
       P_n'''(x)=(2n-1)*P_(n-1)''(x)+P_(n-2)'''(x)  the third derivative
       ...
   a value P_n(x) and P_n(x)' we have from initial for 1 <= n <= Nmax
   the previous values p_(nm) for (m-1) must be in memory }

 Procedure ToClcRecurP ( m  : Byte ) ;
 Var
   k  : Byte ;
   n  : Byte ;
 Begin
   If  m < 2  Then  k:=2  Else  k:=m;
   For n:=1 To MaxIntHarm+1 Do { in  pl^[n]  are  p_(nm)(z/r) }
     pl^[n]:=ul^[n]; { in ul^[n] are the previous derivatives }
   For n:=1 To k-1 Do  ul^[n]:=0.0;
   For n:=k To MaxIntHarm+1 Do
     Begin
        ul^[n]:=(2*n-1)*pl^[n-1];
        If  ( n > (k+1) )
          Then { in  ul^[n]  are  d(p_(nm)(z/r))/d(z/r) }
            ul^[n]:=ul^[n]+ul^[n-2];
     End;
 End;

 { for  cos^m(phi)*(cos or sin)(m*lambda)
   thus we have two simple recurrent formulae
   cos^m(phi)*cos(m*lambda) =
       = cos^(m-1)(phi)*cos((m-1)*lambda)*(x/r) -
       - cos^(m-1)(phi)*sin((m-1)*lambda)*(y/r) ;
   cos^m(phi)*sin(m*lambda) =
       = cos^(m-1)(phi)*sin((m-1)*lambda)*(x/r) +
       + cos^(m-1)(phi)*cos((m-1)*lambda)*(y/r) }

 Procedure ToClcRecurXYm ;
 Var
   xac  : Extended ;
   yac  : Extended ;
   dxx  : Extended ;
   dxy  : Extended ;
   dyx  : Extended ;
   dyy  : Extended ;
 Begin
   xac:=xam*xr-yam*yr; { xr is x/r , xac = cos^m(phi)*cos(m*lambda) }
   yac:=yam*xr+xam*yr; { yr is y/r , yac = cos^m(phi)*sin(m*lambda) }
   dxx:=xax*xr+xam-yax*yr; { partial derivative with respect to x/r }
   dxy:=xay*xr-yay*yr-yam; { partial derivative with respect to y/r }
   dyx:=yax*xr+yam+xax*yr; { partial derivative with respect to x/r }
   dyy:=yay*xr+xay*yr+xam; { partial derivative with respect to y/r }
   xam:=xac; { cos^m(phi)*cos(m*lambda) for the current m }
   yam:=yac; { cos^m(phi)*sin(m*lambda) for the current m }
   xax:=dxx; { ...*cos(m*...) partial derivative with respect to x/r }
   xay:=dxy; { ...*cos(m*...) partial derivative with respect to y/r }
   yax:=dyx; { ...*sin(m*...) partial derivative with respect to x/r }
   yay:=dyy; { ...*sin(m*...) partial derivative with respect to y/r }
 End;

 { to calculate
   recurrent value of the functions for the next value m }

 Procedure ToClcRecurM ( m  : Byte ) ;
 Begin
   If  m = 0  Then  Exit ; { initial values are already }
   ToClcRecurP(m); { for  p_(nm)(x)=d^m(P_n(x))/dx^m }
   ToClcRecurXYm ; { for  cos^m(phi)*(cos or sin)(m*lambda) }
 End;

 { the first part of single potential
      (fm/r)*(r_0/r)^n }

 Procedure UPartOne ( n  : Byte ; Var  r : Extended ; Var  u : TVect3 ) ;
 Var
   i  : Byte ;
   q  : Extended ;
 Begin
   r:=rl^[n]; { (fm/r)*(r_0/r)^n }
   q:=(n+1)*rl^[n-1]*GeoR0; { d((fm/r)*(r_0/r)^n)/d(1/r) }
   For i:=1 To 3 Do  u[i]:=q*dr[i]; { d((fm/r)*(r_0/r)^n)/d(1/r)*d(1/r)/dr }
 End;

 { the second part of single potential
       p_(nm) -> d^m(P_n(z/r))/d(z/r)^m }

 Procedure UPartTwo ( n  : Byte ; Var  p : Extended ; Var  v : TVect3 ) ;
 Var
   i  : Byte ;
 Begin
   p:=pl^[n]; { p_(nm)(z/r) for current m }
   For i:=1 To 3 Do
     v[i]:=ul^[n]*dz[i]; { d(p_(nm)(z/r))/d(z/r)*d(z/r)/(d x,y,z) }
 End;

 { the third part of single potential
      +Cnm*(1-z^2/r^2)^(m/2)*cos(m*lambda)
      +Snm (1-z^2/r^2)^(m/2)*sin(m*lambda)
   simple notation
      Xm = (1-z^2/r^2)^(m/2)*cos(m*lambda) 
      Ym = (1-z^2/r^2)^(m/2)*sin(m*lambda) }

 Procedure UPartTre ( c , s  : Extended ;
                      Var  o : Extended ;
                      Var  w : TVect3 ) ;
 Var
   i  : Byte ;
   x  : Extended ;
   y  : Extended ;
 Begin
   o:=c*xam+s*yam; { Cnm*Xm+Snm*Ym  below are the derivatives }
   x:=c*xax+s*yax; { Cnm*d(Xm)/d(x/r)+Snm*d(Ym)/d(x/r) }
   y:=c*xay+s*yay; { Cnm*d(Xm)/d(y/r)+Snm*d(Ym)/d(y/r) }
   For i:=1 To 3 Do { in dx[.] d(x/r)/d(x,y,z) }
     w[i]:=x*dx[i]+y*dy[i]; { in dy[.] d(y/r)/d(x,y,z) }
 End;

 { to add forces for current m and for m <= n <= N
   we have three part in ptential function
   the first
       (fm/r)*(r_0/r)^n
   the second
       p_(nm) -> d^m(P_n(z/r))/d(z/r)^m
   the third
       Cnm*(1-z^2/r^2)^(m/2)*cos(m*lambda) +
     + Snm (1-z^2/r^2)^(m/2)*sin(m*lambda)
   corrections due to solid body tides are done by CoefWithTide proc }

 Procedure ToAddForceM ( m  : Byte ;  Var  f  : TVect3 ) ;
 Var
   i,k,n  : Byte ;
   c,s    : Extended ;
   r,p,o  : Extended ; { three part of potential }
   u,v,w  : TVect3 ; { derivatives of each part with respect to x,y,z }
 Begin
   If  m < 2  Then  k:=2  Else  k:=m;
   For n:=k To MaxIntHarm Do
     Begin
       UPartOne(n,r,u); { for (fm/r)*(r_0/r)^n }
       UPartTwo(n,p,v); { p_(nm) -> d^m(P_n(z/r))/d(z/r)^m }
       ToGetTwoCoef(n,m,nhcs,c,s); { current values Cnm Snm from UnHarMod }
       CoefWithTide(n,m,c,s); { with deviations due to tides from UnForceT }
       UPartTre(c,s,o,w); { [C+Snm]*(1-z^2/r^2)^(m/2)*[cos+sin](m*lambda) }
       For i:=1 To 3 Do
         f[i]:=f[i]+u[i]*p*o+r*v[i]*o+r*p*w[i]; { current n,m force }
     End;
 End;

 { to calculate the Forces due to all
   zonal tesseral and sectorial harmonics of the Earth potential
   with changing between
   CRS (Celestial Reference System) and TRS (Terrestrial Reference System)
   SelTerM for CRS -> TRS , TerSelM for TRS -> CRS , matrixes from UnForTyp }

 Procedure ForceDueToGeoHarm ( VectXCur           : TVect3 ;
                               Var
                                 ForceCur         : TVect3 ) ;
 Var
   i  : Byte ;
   j  : Byte ;
   m  : Byte ;
   f  : TVect3 ; { for the forces in TRS }
 Begin
   GetMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
   GetMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   GetMem(rl,SizeOf(TMaxPow)); { for (r_0/r)^n }
   nhcs:=0; { initial count for harmonic coefficients }
   ToTransToTerra(VectXCur); { x y z 1/r  in TRS }
   ToClcSimpleDer ; { x/r y/r z/r 1/r^2 and simple partial derivatives }
   ToClcInitRecur ; { initial values for recurrent formulae , m =0 }
   For i:=1 To 3 Do  f[i]:=0.0; { initial nullo forces in TRS }
   m:=0;
 Repeat
   ToClcRecurM(m); { recurrent value for the next m }
   ToAddForceM(m,f); { to add forces for current m and for m <= n <= N }
   m:=m+1;
 Until  m > MaxIntHarm ;
   For i:=1 To 3 Do
     For j:=1 To 3 Do { the forces from TRS to CRS }
       ForceCur[i]:=ForceCur[i]+TerSelM[i,j]*f[j];
   FreeMem(rl,SizeOf(TMaxPow)); { for (r_0/r)^n }
   FreeMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   FreeMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
 End;

 END.
