
 { the procedures to help to define model and data }

 UNIT UnHarMod ;

 INTERFACE

 Const
   MaxPowHarm    =   70 ;
   MaxZonHarm    =   70 ;
   MaxHarmCoef   = 2701 ;
   MaxEilHarm    =   16 ;

 Type
   THarmCoef     = Record { numerical value of the coefficients }
                     nh  : Byte ;
                     mh  : Byte ;
                     ch  : Extended ;
                     sh  : Extended ;
                   End;
   THarmModel    = Array[1..MaxHarmCoef] of THarmCoef ;
   THarModPtr    = ^THarmModel ;

 Const
   BooHarmMem    : Boolean  = FALSE ; { no coefficients in memory }

 Var
   MaxIntHarm    : Byte ; { number of the harmonics to integrate }
   GemParmValue  : Array [1..4] of Extended ;
   ZonHarmValue  : Array [1..MaxZonHarm] of Extended ;
   AllHarmValue  : THarModPtr ;
   EilHarmValue  : Array [1..MaxEilHarm] of Extended ;

 Procedure ToObtainGemParmValues ( strharmod  : String ) ;

 Procedure ToObtainZonHarmValues ( strharmod  : String ) ;

 Procedure ToObtainAllHarmValues ( strharmod  : String ) ;

 Procedure ToWriteAboutHarmModel ( strharmod  : String ) ;

 Procedure ToObtainEilHarmValues ;

 Procedure ToReplacEilHarmValues ;

 Procedure ToLimitHarmNumber ( nf  : Byte ) ;

 Procedure ToGetTwoCoef ( n , m  : Byte ;
                          Var  nc  : Word ;
                          Var  c , s  : Extended ) ;

 IMPLEMENTATION

 Uses
   UnConTyp , { for GeoFM }
   UnForPrt , { for PrtFile }
   UnForFun ; { to work with complex numbers }

 { to obtain from the file with the name strharmod
   the values of the coefficients fm and r0 }

 Procedure ToObtainGemParmValues ( strharmod  : String ) ;
 Var
   fg  : Text ;
   fm  : Extended ;
   re  : Extended ;
 Begin
   Assign(fg,strharmod);
   ReSet(fg);
   ReadLn(fg);
   ReadLn(fg);
   ReadLn(fg,re); { for Earth equatorial radius in m}
   ReadLn(fg,fm); { for geocentric gravitational const in m^3/s^2 }
   GemParmValue[1]:=1.0e-9*fm; { to km^3/s^2 }
   GemParmValue[2]:=1.0e-3*re; { to km }
   GeoFM:=GemParmValue[1];
   GeoR0:=GemParmValue[2];
   Close(fg);
 End;

 { to obtain the values of the coefficients of the zonal harmonics }

 Procedure ToObtainZonHarmValues ( strharmod  : String ) ;
 Var
   GemFile             : Text ;
   i,m,n               : Byte ;
   r,JHarm             : Extended ;
 Begin
   Assign(GemFile,strharmod);
   ReSet(GemFile);
   For n:=1 To MaxZonHarm Do  ZonHarmValue[n]:=0;
   For i:=1 To 9 Do  ReadLn(GemFile);
   Repeat
     ReadLn(GemFile,n,m,JHarm);
     r:=n; { to unnormalize the value of coefficients }
     If  m = 0
       Then
         ZonHarmValue[n]:=-1.0e-6*Sqrt(2*r+1)*JHarm;
   Until  m > 0 ;
   Close(GemFile);
   MaxIntHarm:=MaxEilHarm;
 End;

 { to obtain the values of the coefficients
      of the zonal , tesseral and sectorial harmonics }

 Procedure ToObtainAllHarmValues ( strharmod  : String ) ;
 Var
   GemFile   : Text ;
   i,k,m,n   : Byte ;
   NumH      : Word ;
   p,q,r,z   : Extended ;
   Cnm,Snm   : Extended ;
 Begin
   If  BooHarmMem  Then  Exit ; { coefficients are already in memory }
   GetMem(AllHarmValue,SizeOf(THarmModel));
   Assign(GemFile,strharmod);
   ReSet(GemFile);
   For NumH:=1 To MaxHarmCoef Do
     With  AllHarmValue^[NumH]  Do
       Begin
         nh:=0;
         mh:=0;
         ch:=0.0;
         sh:=0.0;
       End;
   For i:=1 To 9 Do  ReadLn(GemFile);
   NumH:=0;
   While  NOT EOF(GemFile)  Do
     Begin
       ReadLn(GemFile,n,m,Cnm,Snm);
       If  ( ( n <= MaxPowHarm ) AND ( m <= MaxPowHarm ) )
         Then { all coefficients zonal tesseral sectorial }
           Begin
             NumH:=NumH+1;
             p:=1;
             z:=n;
             r:=n-m;
             k:=2*m;
             For i:=1 To k Do  p:=p/(r+i);   { (n-m)!/(n+m)! }
             If  m = 0 { to unnormalize the value of coefficients }
               Then { zonal harmonics }
                 q:=Sqrt(2*z+1) { Sqrt(2n+1) }
               Else { other harmonics m > 0 }
                 q:=Sqrt(2*(2*z+1)*p); { Sqrt(2*(2*n+1)*(n-m)!/(n+m)!) }
             With  AllHarmValue^[NumH]  Do
               Begin
                 nh:=n;
                 mh:=m;
                 ch:=+1.0e-6*q*Cnm;
                 sh:=+1.0e-6*q*Snm;
               End;
           End;
     End;
   Close(GemFile);
   MaxIntHarm:=n;
   BooHarmMem:=TRUE;
 End;

 Procedure ToWriteAboutHarmModel ( strharmod  : String ) ;
 Var
   nharm  : Word ;
 Begin
   WriteLn(PrtFile,'model from the file  ',strharmod);
   WriteLn(PrtFile,'geocentric gravitational constant',
                   GemParmValue[1]:15:5,'  km**3/s**2');
   WriteLn(PrtFile,'   equatorial radius of the Earth',
                   GemParmValue[2]:15:5,'  km');
   WriteLn(PrtFile,'parametr     c',GemParmValue[3]:25:18,'  km');
   WriteLn(PrtFile,'parametr sigma',GemParmValue[4]:25:18);
   WriteLn('coefficients for zonal harmonics');
   WriteLn(' ':9,'n',' ':10,'1e6*Jn');
   WriteLn(PrtFile,'coefficients for zonal harmonics');
   WriteLn(PrtFile,' ':9,'n',' ':10,'1e6*Jn');
   For nharm:=2 To MaxZonHarm Do
     If  nharm <= MaxEilHarm
       Then
         Begin
           WriteLn(nharm:10,1.0e6*ZonHarmValue[nharm]:20:10);
           WriteLn(PrtFile,nharm:10,1.0e6*ZonHarmValue[nharm]:20:10,
                                    1.0e6*EilHarmValue[nharm]:30:18);
         End;
   WriteLn('coefficients for tesseral and sectorial harmonics');
   WriteLn('    n    m',' ':10,'1e6*Cnm   ',' ':10,'1e6*Snm');
   WriteLn(PrtFile,'coefficients for tesseral and sectorial harmonics');
   WriteLn(PrtFile,'    n    m',' ':20,'1e6*Cnm   ',' ':20,'1e6*Snm');
   For nharm:=2 To MaxHarmCoef Do
     With  AllHarmValue^[nharm]  Do
       If  ( ( ( 0 < nh ) AND ( nh < 11 ) )
             AND ( ( 0 < mh ) AND ( mh < 11 ) ) )
         Then
           Begin
             WriteLn(nh:5,mh:5,
                     1.0e6*ch:30:15,1.0e6*sh:30:15);
             WriteLn(PrtFile,nh:5,mh:5,
                             1.0e6*ch:30:25,1.0e6*sh:30:25);
           End;
 End;

 { to obtain simple parameters  c , sigma , J'n
   of the intermediate generalised two fixed centre problem }

 Procedure ToObtainEilHarmValues ;
 Var
   n        : Byte ;
   p,q,r    : Extended ;
   x,y,a,b  : TCompValue ;
   xa,yb,c  : TCompValue ;
   rz       : Extended ;
   zj2,zj3  : Extended ;
   cj2,sig  : Extended ;
 Begin
   rz:=GemParmValue[2];
   zj2:=ZonHarmValue[2];
   zj3:=ZonHarmValue[3];
   r:=zj3/(2*zj2);
   cj2:=rz*Sqrt(zj2-Sqr(r));
   sig:=r*rz/cj2;
   GemParmValue[3]:=cj2;
   GemParmValue[4]:=sig;
   q:=cj2/rz;
   r:=q;
   x.rval:=+1;
   x.ival:=+sig;
   y.rval:=+1;
   y.ival:=-sig;
   a.rval:=+sig;
   a.ival:=+1;
   b.rval:=+sig;
   b.ival:=-1;
   MultComplex(x,a,xa); { from UnForFun }
   MultComplex(y,b,yb);
   x:=xa;
   y:=yb;
   For n:=1 To MaxEilHarm Do
     Begin
       c.rval:=x.rval+y.rval;
       c.ival:=x.ival+y.ival;
       EilHarmValue[n]:=-0.5*r*c.rval;
       r:=r*q;
       MultComplex(x,a,xa);
       MultComplex(y,b,yb);
       x:=xa;
       y:=yb;
     End;
   MaxIntHarm:=MaxEilHarm;
 End;

 Procedure ToReplacEilHarmValues ;
 Var
   n  : Byte ;
 Begin
   MaxIntHarm:=MaxEilHarm;
   For n:=1 To MaxIntHarm Do
     ZonHarmValue[n]:=EilHarmValue[n];
 End;

 Procedure ToLimitHarmNumber ( nf  : Byte ) ;
 Begin
   If  nf > 1
     Then
       MaxIntHarm:=nf
     Else
       MaxIntHarm:=0;
 End;

 { to get current values
   of harmonic coefficients Cnm , Snm
      from AllHarmValue^[NumHarmCoef] }

 Procedure ToGetTwoCoef ( n , m  : Byte ;
                          Var  nc  : Word ;
                          Var  c , s  : Extended ) ;
 Var
   nx  : Byte ;
   mx  : Byte ;
 Begin
   Repeat
     nc:=nc+1;
     nx:=AllHarmValue^[nc].nh;
     mx:=AllHarmValue^[nc].mh;
   Until ( ( ( nx = n ) AND ( mx = m ) ) OR ( nx = 0 ) ) ;
   c:=AllHarmValue^[nc].ch;
   s:=AllHarmValue^[nc].sh;
 End;

 END.
