
 UNIT unharzot ;

 INTERFACE

 Const
   maxp  = 219 ;

 Type
   TPol  = Array[0..maxp] of Extended ;

 Procedure RecurentFormula ; { for Legandre polinoms }

 IMPLEMENTATION

 uses
   unharstr ; { function FloatWithPower }

 Var
   pnul,pone,ptwo,pdif  : TPol ;

 Procedure NulloPol ; { for the initial value }
 Var
   nz  : Byte ;
 Begin
   For nz:=0 To maxp Do
     Begin
       pnul[nz]:=0.0;
       pone[nz]:=0.0;
       ptwo[nz]:=0.0;
       pdif[nz]:=0.0;
     End;
   pnul[0]:=1.0; { P0(z)=1 }
   pone[1]:=1.0; { P1(z)=1 }
 End;

 Procedure RocirPol ( np  : Byte ) ; { to shift polinoms }
 Var
   nz  : Byte ;
 Begin
   For nz:=0 To np Do
     Begin
       pnul[nz]:=pone[nz];
       pone[nz]:=ptwo[nz];
       ptwo[nz]:=0.0;
       pdif[nz]:=0.0;
     End;
 End;

 Procedure RecurPol ( np  : Byte ) ; { simple recurent }
 Var
   nz  : Byte ;
   rn  : Extended ;
   ra  : Extended ;
   rb  : Extended ;
 Begin
   rn:=1.0/np;
   ra:=(np-1)*rn;
   rb:=(2*np-1)*rn;
   For nz:=0 To np-2 Do
     ptwo[nz]:=-ra*pnul[nz];
   For nz:=1 To np Do
     ptwo[nz]:=ptwo[nz]+rb*pone[nz-1];
   For nz:=1 To np Do    { for all element }
     pdif[nz]:=ptwo[nz]; { to differentite }
 End;

 Procedure PolWrite ( n  : Byte ;  m  : byte ;  p  : tpol ) ;
 Var
   i  : byte ;
   k  : Byte ;
 Begin
   WriteLn(n:5,m:5,'  P_',n,'^',m,'(z)');
   i:=0; { initial number of terms }
   For k:=0 To maxp Do
     If  Abs(p[k]) > 1.0e-6
       Then
         i:=i+1; { the next term }
   WriteLn(i:10,'  terms ');
   i:=0;
   For k:=0 To maxp Do
     If  Abs(p[k]) > 1.0e-6  { function unharstr }
       Then
         begin
           i:=i+1;
           WriteLn(i:10,' ':10,FloatWithPower(p[k],30),'  * z^',k);
         end;
 End;

 { to differentiate  d^m(P_n(z))/dz^m m=1,2,...,n }

 procedure PolDifun ( n  : byte ) ; { to differentiate }
 var
   i  : byte ;
   m  : byte ; { count }
 begin
   for m:=1 to n do { polinom by polinom to differentiate }
     begin                  { d(a_i*z^i)/dz=i*a_i*z^(i-1) }
       for i:=0 to maxp do           { the next polinom }
         pdif[i]:=(i+1.0)*pdif[i+1]; { with derivations }
       PolWrite(n,m,pdif);
     end;
 end;

 { to obtain the formula for each Legandre polinom
   or the numerical values of the coefficients an
   in the form
   Pn(z)=a0+a1*z+a2*z**2+a3*z**3+a4*z**4+...+an*z**n
   by the use recurent formula
   n*Pn(z)=(2*n-1)*z*P(n-1)(z)-(n-1)*P(n-2)(z),  P0(z)=1, P1(z)=z }
   
 Procedure RecurentFormula ;
 Var
  np  : Byte ;
 Begin
   NulloPol ;
   For np:=2 To maxp Do
     Begin
       RecurPol(np);
       if  np = maxp
         then
           begin
             PolWrite(np,0,ptwo); { polinom }
             PolDifun(np); { with array pdif[..] }
           end;
       RocirPol(np);
     End;
 End;

 END.
