
 { to differentiate P_n(x)
   0 <= m <= nmax , m <= n <= nmax but n >=2
   d^m(P_n(x))/d(x)^m  -> only x as variable }

 UNIT UnPolFun ;

 INTERFACE

 { to check derivatives
   of Legandre polinoms d^m/dx^m(P_n(x)
   d^m/dx^m [(1-2*\alpha*x+\alpha^2)^(-1/2)]
   = Sum (i=0) (i=infinity) [ \alpha^i*[d^m/dx^m P_i(x)] ] }

 Procedure DerPolLegandre ;

 { to check recurrence with the values
   d^(n-1)/dx^(n-1)[P_n(x)] = 1*3*5*...*(2*n-1)*x 
      and  d^n/dx^n[P_n(x)] = 1*3*5*...*(2*n-1) }

 Procedure DerPolegandrem ;

 IMPLEMENTATION

 Uses
   unforprt , { for PrtFile to print }
   unharstr ; { function FloatWithPower }

 const
   maxarr  = 777 ;

 Type
   TMaxPow     = Array[1..maxarr+1] of Extended ;
   TMaxPowPtr  = ^TMaxPow ; { for arrays }

 Var
   pl,ul       : TMaxPowPtr ; { for Legandre polinoms and r0/r pointer }


 { for m = 0
   initial values for recurrence process
   P_n(x)=((2n-1)/n)*x*P_(n-1)(x)-(n-1)*P_(n-2)(x)
   P_n'(x)=n*P_(n-1)(x)+x*P_(n-1)'(x)
   P_n'(x)=(2n-1)*P_(n-1)(x)+P_(n-2)'(x) }

 Procedure ToClcInitRecur ( x  : extended ) ; { argument x }
 Var
   n  : word ;
   q  : Extended ;
 Begin  { argument x is known }
   pl^[1]:=x;              { P_1(x)=1 Legandre polinom m=0 }
   pl^[2]:=1.5*Sqr(x)-0.5; { P_2(x)=3/2*x*x-1/2 Legandre polinom }
   ul^[1]:=1.0;            { derivative P_1'(x)=1 m=0 }
   ul^[2]:=3.0*x;          { derivative P_2'(x)=3*x }
   For n:=3 To maxarr+1 Do   { maxarr array dimension }
     Begin
       q:=n; { Legandre polinom and derivative for m=0 }
       pl^[n]:=((2.0*q-1.0)/q)*x*pl^[n-1]-((q-1.0)/q)*pl^[n-2]; { P_n(x) }
       ul^[n]:=q*pl^[n-1]+x*ul^[n-1]; { P_n'(x) the first derivative }
     End; { by using  P_n'(x)=n*P_(n-1)(x)+x*P_(n-1)'(x) }
 End;

 { to calculate
   recurrent value of the functions for the next value m
   we need the value
   p_(nm)(x)=d^m(P_n(x))/dx^m and p_(nm)'(x)=d^(m+1)(P_n(x))/dx^(m+1)
   for this calculations it is convenient recurrent formula
       P_n'(x)=(2n-1)*P_(n-1)(x)+P_(n-2)'(x)   the first derivative
       P_n''(x)=(2n-1)*P_(n-1)'(x)+P_(n-2)''(x)  the second derivative
       P_n'''(x)=(2n-1)*P_(n-1)''(x)+P_(n-2)'''(x)  the third derivative
       ...
   a value P_n(x) and P_n(x)' we have from initial for 1 <= n <= Nmax
   the previous values p_(nm) for (m-1) must be in memory }

 Procedure ToClcRecurP ( m  : word ) ;
 Var
   k  : word ;
   n  : word ;
   q  : extended ;
 Begin
   If  m < 2  Then  k:=2  Else  k:=m;
   For n:=1 To maxarr+1 Do   { in  pl^[n]  are  p_(nm)(z/r) }
     pl^[n]:=ul^[n];       { in ul^[n] are the previous derivatives }
   For n:=1 To k-1 Do
     ul^[n]:=0.0;          { to nullo }
   For n:=k To  maxarr+1 Do
     Begin
        q:=n;
        ul^[n]:=(2.0*q-1.0)*pl^[n-1];
        If  ( n > (k+1) )
          Then            { in  ul^[n]  are  d(p_(nm)(z/r))/d(z/r) }
            ul^[n]:=ul^[n]+ul^[n-2];
     End;
 End;

 { to calculate
   recurrent value of the functions for the next value m }

 Procedure ToClcRecurM ( m  : word ) ;
 Begin
   If  m = 0  Then  Exit ; { initial values are already }
   ToClcRecurP(m); { for  p_(nm)(x)=d^m(P_n(x))/dx^m }
 End;

 { Sum (i=0) (i=infinity)
   [ \alpha^i*[d^m/dx^m P_i(x)] ] }

 function vals ( m  : word ;  a ,f  : extended ) : extended ;
 var
   i  : word ;     { count }
   r  : extended ; { \alpha in power }
   s  : extended ; { sum value }
   b  : boolean ;  { to compare differencies }
 begin
   r:=1.0;   { initial for power \alpha }
   s:=0.0;   { initial to sum }
   b:=False; { initial value }
   i:=0;
   While  ( ( i < maxarr ) and ( not b ) )  do
     begin            { for the current order }
       i:=i+1;
       r:=a*r;        { \alpha to the next power }
       s:=s+r*pl^[i]; { the next term to sum }
       b:= ( abs((s-f)/f) < 1.0e-6 ) ; { to compare values }
       if  b  then                  { values are closed }
         if  ( ( f < 1.0e13 ) and ( Abs(s) < 1.0e13 ) )
           then
             writeln(prtfile,m:5,i:5,' ',s:35:16,' ',
                             FloatWithPower(((s-f)/f),16),
                             FloatWithPower(pl^[i],12))
           else
             writeln(prtfile,m:5,i:5,' ':15,FloatWithPower(s,16),' ':4,
                             FloatWithPower(((s-f)/f),16),
                             FloatWithPower(pl^[i],12));
     end;
   if  ( not b )  then { to maximum number }
     if  ( ( f < 1.0e13 ) and ( Abs(s) < 1.0e13 ) )
       then
         writeln(prtfile,m:5,i:5,' ',s:35:16,' ',
                         FloatWithPower(((s-f)/f),16),
                         FloatWithPower(pl^[i],12))
       else
         writeln(prtfile,m:5,i:5,' ':15,FloatWithPower(s,16),' ':4,
                         FloatWithPower(((s-f)/f),16),
                         FloatWithPower(pl^[i],12));
   vals:=s; { our result our sum }
 end;

 { d^m/dx^m [(1-2*\alpha*x+\alpha^2)^(-1/2)]
   = (2*m-1)!! * \alpha^m * (1-2*\alpha*x+\alpha^2)^(-(2*n+1)/2) }

 function funv ( m  : word ; a , x  : extended ) : extended ;
 var
   i  : word ;
   f  : extended ;
   q  : extended ;
   p  : extended ;
   r  : extended ;
 begin
   q:=1.0;             { initial 1.0 }
   for i:=1 to m do    { for (2*m-1)!! }
     q:=(2.0*i-1.0)*q; { = 1*3*5*...*(2*m-1) }
   r:=1.0;             { initial 1.0 }
   for i:=1 to m do    { for \alpha^m a for \alpha }
     r:=a*r;           { power by power for \alpha }
   { for function }
   p:=1.0/(1.0-2.0*a*x+sqr(a)); { 1/(1-2*\alpha*x+\alpha^2) }
   f:=sqrt(p);                   { (1-2*\alpha*x+\alpha^2)^(-1/2) }
   for i:=1 to m do    { to power -(2*n+1)/2 }
     f:=p*f;           { power by power }
   funv:=q*r*f; { our result }
 end;

 { to check derivatives
   of Legandre polinoms d^m/dx^m(P_n(x)
   d^m/dx^m [(1-2*\alpha*x+\alpha^2)^(-1/2)]
   = Sum (i=0) (i=infinity) [ \alpha^i*[d^m/dx^m P_i(x)] ] }

 Procedure DerPolLegandre ;
 Var
   i  : word ;
   m  : word ;
   n  : word ;
   a  : extended ; { parameter \alpha < 0.999 }
   x  : extended ; { argument x ( -1 < x < +1 ) }
   f  : extended ; { function }
   s  : extended ; { sum with derivatives }
   r  : extended ;
 Begin
   GetMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
   GetMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   n:=36;     { for the derivatives }
   a:=+0.90;  { parameter \alpha }
   x:=+0.99;  { argument x }
   writeln(prtfile,'parameter \alpha value',a:10:3);
   writeln(prtfile,'parameter    x   value',x:10:3);
   r:=1.0;   { initial for power \alpha }
   for i:=37 to 47 do
     begin
       r:=a*r;
       writeln(prtfile,i:5,a:10:2,r:16:6); { unharstr }
     end;
   ToClcInitRecur(x); { initial values for recurrent formulae , m =0 }
   m:=1;              { the first derivative }
 Repeat            { derivative by derivative }
   f:=funv(m,a,x); { d^m/dx^m [(1-2*\alpha*x+\alpha^2)^(-1/2)] }
   if  f < 1.0e12
     then
       writeln(prtfile,m:5,' ':5,' ',f:35:16)
     else
       writeln(prtfile,m:5,' ':5,' ':15,FloatWithPower(f,16));
   ToClcRecurM(m); { recurrent value for the next m result to pl^[..] }
   s:=vals(m,a,f); { sum [ \alpha^i*[d^m/dx^m P_i(x)] ] }
   m:=m+1;         { the next derivative }
 Until  m > n ;    { global uncontyp }
   FreeMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   FreeMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
 End;

 { to check recurrence with the values
   d^(n-1)/dx^(n-1)[P_n(x)] = 1*3*5*...*(2*n-1)*x
      and  d^n/dx^n[P_n(x)] = 1*3*5*...*(2*n-1) }

 Procedure DerPolegandrem ;
 Var
   i  : word ;
   m  : word ;
   n  : word ;
   x  : extended ; { argument x ( -1 <= x <= +1 ) }
   q  : extended ; { value temporary }
   r  : extended ; { value temporary }
 Begin
   GetMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
   GetMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   n:=720;     { for the derivatives }
   x:=+0.99;   { argument x }
   writeln(prtfile); { global text file from unforprt }
   writeln(prtfile,'to check d^n/dx^n[P_n(x)] = 1*3*5*...*(2*n-1)');
   ToClcInitRecur(x); { initial values for recurrent formulae , m =0 }
   m:=1;              { the first derivative }
 Repeat            { derivative by derivative }
   q:=1.0;             { initial 1.0 }
   for i:=1 to m do    { for (2*m-1)!! }
     q:=(2.0*i-1.0)*q; { = 1*3*5*...*(2*m-1) }
   r:=x*q;             { = 1*3*5*...*(2*m-1) * x value }
   if  q < 1.0e19
     then
       begin
         writeln(m:3,x:6:2,' ',q:32:2,r:32:2);
         writeln(' ':10,ul^[m]:32:2,pl^[m]:32:2);
         writeln(prtfile,m:3,x:6:2,' ',q:32:2,r:32:2);
         writeln(prtfile,' ':10,ul^[m]:32:2,pl^[m]:32:2);
       end
     else
       begin
         writeln(m:3,x:6:2,
                 ' ':6,GloatWithPower(q,25),
                 ' ':5,GloatWithPower(r,25));
         writeln(' ':15,GloatWithPower(ul^[m],25),
                 ' ':5,GloatWithPower(pl^[m],25));
         writeln(prtfile,m:3,x:6:2,
                         ' ':6,GloatWithPower(q,25),
                         ' ':5,GloatWithPower(r,25));
         writeln(prtfile,' ':15,GloatWithPower(ul^[m],25),
                         ' ':5,GloatWithPower(pl^[m],25));
       end;
   writeln(' ':28,GloatWithPower((ul^[m]-q),12),
           ' ':18,GloatWithPower((pl^[m]-r),12));
   writeln(prtfile,' ':28,GloatWithPower((ul^[m]-q),12),
                   ' ':18,GloatWithPower((pl^[m]-r),12));
   ToClcRecurM(m); { recurrent value for the next m result to pl^[..] }
   m:=m+1;         { the next derivative }
 Until  m > n ;    { global uncontyp }
   FreeMem(ul,SizeOf(TMaxPow)); { for P_n'(x) }
   FreeMem(pl,SizeOf(TMaxPow)); { for P_(n)(x) }
 End;

 END.
